#' Generation of Distance Matrices with Error
#'
#' @param n # of stimuli
#' @param nG # of subjects
#' @param ndim # of dimensions
#' @param stderror The standard deviation of error to be added.
#' @param X n x ndim Stimulus Configuration Matrix or NULL.
#' @param W nG x ndim  Subject Weight Matrix or NULL
#' @param print = 1 to print the result
#'
#' @details
#' When X is NULL a n x ndim matrix will be generated using rnorm. \cr
#' When W is NULL a nG x ndim matrix will be generated using runif
#' and normalized so that \eqn{diag(t(W)%*%W)=I}. \cr
#'
#' Let D_k be the weighted Euclidean distance calculated from X and W[k,].
#' The O_k will be calculated as \cr
#'  \deqn{ O_k = D_k + stderror*var(vech(D_k)) }
#'
#' @return
#' A list of \cr
#'  vO:  n*(n-1)/2 x nG array consisting of O_k, k=1,2, ..., nG \cr
#'  vD:  n*(n-1)/2 x nG array consisting of O_k, k=1,2, ..., nG \cr
#'  X:  n x ndim configuration matrix \cr
#'  W:  nG x ndim matrix of subject weights \cr
#'  stderror:  standard deviation of the addtive error
#'
#' @examples
#'
#' set.seed(1701)
#' n <- 15; nG <- 5; ndim <- 2
#' resgen1 <- gendatamds( n, nG=3, ndim=2, stderror=0.2 )
#' O <- vechinv(resgen1$vO, nodiag=1, array=1)  # This is n x n x nG.
#'
#' X <- matrix( c( 0,4, 0,0, 3,0, 3,4, 1.5,2 ), ,2, byrow=1 )
#' dimnames(X) <- list(paste("s",1:nrow(X),sep=""),paste("d",1:ncol(X),sep=""))
#' W <- matrix( c( 1,1, 2,1, 1,2 ),,2, byrow=1 )
#' dimnames(W) <- list(paste("w",1:nrow(W),sep=""),paste("d",1:ncol(W),sep=""))
#' resgen2 <- gendatamds( X=X, W=W, stderror=0.2 )
#' O2 <- vechinv(resgen2$vO, nodiag=1, array=1)  # This is n x n x nG.
#'
#' @export
#'

gendatamds <- function( n, nG=1, ndim=2, stderror=0.2, X=NULL, W=NULL
                        , print=0 ){
 # genration of Distance matrices
 # Shin-ichi Mayekawa
 # 20151127
 #

 # generate X and W
 if( is.null(X) ){
  X=matrix( rnorm(n*ndim), n, ndim )
  rownames(X)=paste("S",1:n,sep="")
  colnames(X)=paste("d",1:ndim,sep="")
 }
 if( is.null(W) ){
  W=matrix( runif(nG*ndim), nG, ndim )
  rownames(W)=paste("subj",1:nG,sep="")
  colnames(W)=paste("d",1:ndim,sep="")
 }

 n=nrow(X); nG=nrow(W); ndim=ncol(X)
 npair=n*(n-1)/2

 # normalize W
 wn=colnames(W)
 W=W%*%Diag(1/sqrt(colSums(W^2)))
 colnames(W)=wn
 # weighted Euclidean distance
 vD=vech(distancew( X, W ),nodiag=1)
 # add scaled error
 stdD=sqrt( diag(cov(vD)*(npair-1)/npair) )
 stderr=.2
 vO=vD + matrix(rnorm(npair*nG),npair,nG )%*%Diag(stderror*stdD)
 vO[vO<0]=0

 if( print ){
  Print( n, nG, ndim, npair )
  Print( X, W )
  if( print >= 2 )
   Print( vO, vD)
 }

 res=named_list( vO, vD, X, W, stderror)
 return( res )

} # gendatamds