indscal <- function( B, nr=2, estm=0, orthX=0, lmax=50, eps=1e-7, print=2
                      , X=NULL ){
 # indscal
 # Shin-ichi Mayekawa
 # 120107,08,09
 # estm : 120116
 #
 # Given nG B_k matrices, this program finds Xc and W_k such that
 #      B_k == Xc diag(W_k)^2 Xc'
 #  Xc and W_k matrices are estimated by INDSCAL algorithm
 # 
 #  Note that 
 #   B_k == X_k X_k'   where X_k = Xc diag(W_k) T_k
 #  Therefore, given nG X_k matrices, we may first calculate B_k matrices,
 #  find Xc and W_k matrices, and then find T_k matrices.
 #
 #  Xc matrix will be normalized so that diag(t(Xc)%*%Xc) = n I
 #   
 #  Dimensions will be ordered according to W[##,]
 #
 # Args:
 #   B     n x n x nG array of scalar products
 #
 #   X     n x nr x nG array of scale values
 #         If X is given, B will be calculated from X.
 #
 #   estm  = 1 to estimate the column menas when X is given
 #
 #   orthX   not yet available
 #
 #
 #
 # Value: as list
 #
 #    X    n x nr
 #    W    nG x nr        containing the diagonal elements of W_k
 #    T    nr x nr x nG   array of T_k matrices
 #    m    nG x nr        column means
 #
 #
 #
 #
 
 getrmse <- function( title, nG, n, nr, B, Xr, Xl, W, rmsep ){
  # make B an array of n x n x nG
  dim(B)=c(n,n,nG)

  rmse=0;
  for( k in 1:nG ){
   TWT=Xl%*%Diag(W[k,])%*%t(Xr)   # W already squared
   rmse=rmse+sum((B[,,k]-TWT)^2)
  }
  rmse=sqrt(rmse/(nG*n*n))
  cat(title, ":", rmse, "\n")
  if( rmsep < rmse ){
    cat(title, ": **** failure !! ****", rmsep - rmse, " \n")
  }
  return( rmse )
 }

 
 # If X is given use it to calculate B matrix.
 Xgiven=0
 if( !is.null(X) ){
  Xgiven=1
  n=dim(X)[1]; nG=dim(X)[3]; nr=dim(X)[2]
  onen=matrix(1,n,1)
  meanX=matrix(0,nG,nr)
  if( estm == 1 ){
   for( k in 1:nG ){
    meanX[k,]=matrix(colMeans(X[,,k]),1,nr)
    X[,,k]=X[,,k]-onen%*%meanX[k,,drop=F]
   }
  }
  B=array(0,c(n,n,nG)) 
  for( k in 1:nG ){
   B[,,k]=X[,,k]%*%t(X[,,k])
  }
 }
 
 # const
 n=ncol(B);
 nG=dim(B)[3]
 
 if( print >= 1 ){
  cat("\n\nWeighted Euclidean MDS by INDSCAL", "\n")
  cat(" # of matrices = ", nG, ",  the size of X = ", n, " x ", nr, "\n")
  if( Xgiven ) cat(" X_k matrices will be converted to B_k matrices... \n")
  cat(" estimation of the column means = ", estm, "\n")
  cat(" orthogonal X = ", orthX, "\n")
  cat(" max # of iterations = ", lmax, ",  eps = ", eps, "\n")
 }

 
 # sum(B) == Theta Lambda Theta'  (see Procrustes.tex)
 sumB=apply(B,1:2,sum)
 ev=eigen(sumB); eva=matrix(ev$values,length(ev$values),1); eve=ev$vectors;
 eva=eva[1:nr,]; eve=eve[,1:nr]
 Xc=eve%*%Diag(sqrt(eva))
 Xl=Xc; Xr=Xc
 W=matrix(1,nG,nr)
 
 converged=0;
 rmse=0;
 for( k in 1:nG ){
  TWT=Xl%*%Diag(W[k,])%*%t(Xr)
  rmse=rmse+sum((B[,,k]-TWT)^2)
 }
 rmsep=sqrt(rmse/(nG*n*n))
 Xlp=Xl; Xrp=Xr; Wp=W

 rmsep=getrmse( "initial", nG, n, nr, B, Xr, Xl, W, rmsep )


 # main iteration

 for( llll in 1:lmax ){
 
  # estimate W:  This W is W^2
  for( k in 1:nG ){
   Wk=solve((t(Xc)%*%Xc) * (t(Xc)%*%Xc) ) %*%
      vecdiag(t(Xc)%*%B[,,k]%*%Xc)  
   W[k,]=t(Wk)
  }
  
  # negative Ws set on the boundary
  W[which(W<0)]=0
  
  if( print >= 4 )
   rmse0=getrmse( "after W", nG, n, nr, B, Xr, Xl, W, rmsep )
 
  
  WW=W%x%matrix(1,n)
  # Print(nrow(WW),ncol(WW))

  # make t(B) a vertically stacked matrix of  nG*n x n
  dim(B)=c(n,n*nG)
  # Print(nrow(B),ncol(B))

  # estimate Xl
  XX=matrix(rep(t(Xr),nG),,nr,byrow=1)*WW    # This is nG*n x nr matrix
  # Print(Xr, matrix(rep(t(Xr),nG),,nr,byrow=1),nrow(XX),ncol(XX))
  Xl=t( solve(t(XX)%*%XX)%*%t(XX)%*%t(B) )
  if( print >= 4 )
   rmse0=getrmse( "after Xl", nG, n, nr, B, Xr, Xl, W, rmse0 )

  # normalize Xl so that diag(t(Xl)%*%Xl) = n I
  dd=diag( sqrt( n/diag(t(Xl)%*%Xl) ) )
  Xl=Xl%*%dd; 
  W=W%*%solve(dd*dd);
  WW=W%x%matrix(1,n)
  if( print >= 4 )
   rmse0=getrmse( "after norm Xl", nG, n, nr, B, Xr, Xl, W, rmse0 )


  # estimate Xr
  XX=matrix(rep(t(Xl),nG),,nr,byrow=1)*WW    # This is nG*n x nr matrix
  Xr=t( solve(t(XX)%*%XX)%*%t(XX)%*%t(B) )
  if( print >= 4 )
   rmse0=getrmse( "after Xr", nG, n, nr, B, Xr, Xl, W, rmse0 )

  # normalize Xr so that diag(t(Xr)%*%Xr) = n I
  dd=diag( sqrt( n/diag(t(Xr)%*%Xr) ) )
  Xr=Xr%*%dd; 
  W=W%*%solve(dd*dd);
  if( print >= 4 ) 
   rmse0=getrmse( "after norm xr", nG, n, nr, B, Xr, Xl, W, rmse0 )

  # mean 
  Xc=(Xl+Xr)/2


  # make B an array of n x n x nG
  dim(B)=c(n,n,nG)
  
  rmse=0;
  for( k in 1:nG ){
   TWT=Xl%*%Diag(W[k,])%*%t(Xr)
   rmse=rmse+sum((B[,,k]-TWT)^2)
  }
  rmse=sqrt(rmse/(nG*n*n))
  rmseimp=rmsep-rmse;
  rmseimpr=rmseimp/(rmsep+1e-7);
  maxadXl=max(abs(Xlp-Xl));
  maxadXr=max(abs(Xrp-Xr));
  maxadW=max(abs(W-Wp));

  if( print >= 2 )
   Print(llll, rmsep, rmse, rmseimp, maxadXl, maxadXr, maxadW)
  
  if ( rmseimpr < eps || maxadW < eps) {
   converged=1;
   break;
  }

  # next iteration;
  Tp=T; Wp=W; Xlp=Xl; Xrp=Xr
  rmsep=rmse;

  
 } # end of llll loop

 
 if ( !converged ) 
  cat("\n Warning(indscal): Iteration did not converge.\n\n")
  

 # square root of W
 W[which(W<0)]=0
 W=sqrt(W)
 
 # normalize Xc so that diag(t(Xc)%*%Xc) = n I
 dd=diag( sqrt( n/diag(t(Xc)%*%Xc) ) )
 Xc=Xc%*%dd;
 W=W%*%solve(dd);

 # sort the dimensions according to W^2
 od=nr+1-order( colSums(W*W) )
 W=W[,od]
 Xc=Xc[,od]
 maxadXlr=max(abs(Xl-Xr))
 
 rmseB=0;
 for( k in 1:nG ){
  TWT=Xc%*%Diag(W[k,]^2)%*%t(Xc)
  rmseB=rmseB+sum((B[,,k]-TWT)^2)
 }
 rmseB=sqrt(rmseB/(nG*n*n))
 
 # find T such that X_k = Xc W_k T_k by orthogonal Procrustes
 T=array(0,c(nr,nr,nG))
 rmseX=NA
 if( Xgiven ){
  for( k in 1:nG ){
  # rotate Xc diag(W_k) to match X_k
   XcW=Xc%*%Diag(W[k,])
   CAAC=t(X[,,k])%*%XcW%*%t(XcW)%*%X[,,k];
   ev=eigen(CAAC); eva=matrix(ev$values,length(ev$values),1); eve=ev$vectors;
   rm(ev); loc=which(eva > 0); eva=eva[loc]; eve=eve[,loc];
   SQR=eve%*%Diag(sqrt(1/eva))%*%t(eve);
   Tk=t(XcW)%*%X[,,k]%*%SQR;
   T[,,k]=Tk
   if( estm == 1 ){
    X[,,k]=X[,,k]+onen%*%meanX[k,,drop=F]
   }
  }
  rmseX=0;
  for( k in 1:nG ){
   XWT=Xc%*%Diag(W[k,])%*%T[,,k] + onen%*%meanX[k,,drop=F]
   rmseX=rmseX+sum((X[,,k]-XWT)^2)
   if( print >= 3 ) Print(k,"/",X[,,k],XWT, digits=2)
  }
  rmseX=sqrt(rmseX/(nG*n*nr))
 }

 if( print >= 1 ){
  cat("\n\nWeighted Euclidean MDS by INDSCAL", "\n")
  cat("  # of matrices = ", nG, ",  the size of X = ", n, " x ", nr, "\n")
  if( Xgiven ) cat(" X_k matrices were converted to B_k matrices... \n")
  cat(" estimation of the column means = ", estm, "\n")
  cat(" # of iterations = ", llll, ",  eps = ", eps, "\n")
  cat(" rmse in terms of B = ", rmseB, ", rmse in terms of X = ", rmseX, "\n")
  cat(" max abs difference between Xl and Xr = ", maxadXlr, "\n")
  cat("\n Common Configuration Matrix and weight matrices: \n\n")
  Print(Xc, W, digits=3)
  if( estm == 1 ){
   cat(" column means \n"); print(meanX)
  }
  cat("\n Rotation matrix of the form:  X_k = X W_k T_k  \n\n")
  Print(T, digits=3)
 }


 res=list(X=Xc, W=W, T=T, rmseB=rmseB, rmseX=rmseX)
 
 return( res )
 
} # end of indscal





seed=213409875
set.seed(seed)


nr=3;
lmax=500; eps=1e-8
print=2


nG=3; n=9; nr=2
errstd=.1
resg=gendataWm( nG, n, nr, genW=2, pcA=1, eqW=0, errstd=errstd )
Call=resg$C
Xall=array(unlist(Call), dim = c(dim(Call[[1]]), length(Call))) 
rm(Call)
Xg=resg$A; Tg=resg$T; Wg=resg$W


res=indscal( B, nr, estm=1, lmax=lmax, eps=eps, print=1, X=Xall )
Xi=res$X
Wi=res$W
Ti=res$T
mi=res$m

