#' INDSCAL
#'
#' @param B Input scalar product array
#' @param nr # of dimensions
#' @param lmax max # of iterations
#' @param eps convergence criterion for B
#' @param print = 1 to print result
#' @param X The initial value of X:   n x ndim
#' @param W The initial value of W:  nG x ndim
#' @param estm = 1 to subtract column mean from X when Xall is given.
#' @param Xall An Array of X_k's, or NULL
#'
#' @details
#' Let B_k, k=1,2, ... , nG, be the scalar product matrices calculated
#' from the distance matrices, D_k, as \cr
#' \deqn{
#'  B_k = -2 J D_k^2 J
#' }
#' where \eqn{J} is the column centering operator. \cr
#' This program find those Xc and diagonal W_k matrices that minimize
#' \deqn{
#'   RSS = tr( (B_k -  Xc diag(W_k)^2 Xc')' (B_k -  Xc diag(W_k)^2 Xc') )
#'   }
#'  Note that, if X_k can be expressed as
#'  \deqn{
#'   X_k = Xc diag(W_k) T_k
#'   }
#'  we have
#'  \deqn{
#'   B_k == X_k X_k' = Xc diag(W_k)^2 Xc'
#'   }
#'  Therefore, given nG X_k matrices, we may first calculate B_k matrices,\cr
#'  find Xc and W_k matrices by INDSCAL, and then find T_k matrices.\cr
#' \cr
#' When Xall is present, it has priority over B. \cr
#' \cr
#'  Xc matrix will be normalized so that diag(t(Xc)%*%Xc) = n I .\cr
#'
#' @return A list of \cr
#' X=Xc, W, T, rmseB, rmseX
#'
#' @examples
#' set.seed(1701)
#' # generate O from X and W
#' X <- matrix( c( 0,4, 0,0, 3,0, 3,4, 1.5,2 ), ,2, byrow=1 )
#' dimnames(X) <- list(paste("s",1:nrow(X),sep=""), paste("d",1:ncol(X),sep=""))
#' X <- standard( X, center=1 )
#' W <- matrix( c( 1,1, 2,1, 1,2 ),,2, byrow=1 )
#' rownames(W) <- paste("sbj",1:nrow(W),sep=""); colnames(W) <- colnames(X)
#' resgen <- gendatamds( X=X, W=W, stderror=0.05, print=2 )
#' O <- vechinv(resgen$vO, nodiag=1, array=1)  # This is n x n x nG.
#' # double center O
#' n <- nrow(X); nG <- nrow(W)
#' J <- diag(n)-(1/n)*matrix(1,n,1)%*%matrix(1,1,n)
#' B <- O
#' for( k in 1:nG ){
#'  B[,,k] <- -0.5*J%*%O[,,k]^2%*%J
#' }
#' Print(O,B, fmt=".2")
#'
#' res1 <- indscal( B, nr=2, print=2 )
#' res2 <- sumscal( B, nr=2, print=2 )
#' res12 <- indscal( B, nr=2, X=res2$X, W=res2$W, print=2 )
#'
#' @references
#' J.D. Carroll & J.-J. Chang, (1970) Analysis of Individual Differences in
#'  Multidimensional scaling via an N-way generalization of "Eckart-Young"
#'   Decomposition. Psychometrika 35: 283-319.
#'
#'
#' @export
#'

indscal <- function( B, nr=2, lmax=500, eps=1e-6, print=2
                    , X=NULL, W=NULL, estm=0, Xall=NULL ){
 # indscal
 # Shin-ichi Mayekawa
 # 120107,08,09
 # estm : 120116
 # orthX deleted: 20150607
 # initial X and W: 20150727
 #
 # Given nG B_k matrices, this program finds Xc and W_k such that
 #      B_k == Xc diag(W_k)^2 Xc'
 #  Xc and W_k matrices are estimated by INDSCAL algorithm
 #
 #  Note that
 #   B_k == X_k X_k'   where X_k = Xc diag(W_k) T_k
 #  Therefore, given nG X_k matrices, we may first calculate B_k matrices,
 #  find Xc and W_k matrices, and then find T_k matrices.
 #
 #  Xc matrix will be normalized so that diag(t(Xc)%*%Xc) = n I
 #
 #  Dimensions will be ordered according to W[##,]
 #
 # Args:
 #   B     n x n x nG array of scalar products
 #
 #   Xall  n x nr x nG array of scale values
 #         If Xall is given, B will be calculated from Xall.
 #
 #   estm  = 1 to estimate the column menas when Xall is given
 #
 #   orthX   not yet available
 #
 #
 #
 # Value: as list
 #
 #    X    n x nr
 #    W    nG x nr        containing the diagonal elements of W_k
 #    T    nr x nr x nG   array of T_k matrices
 #    m    nG x nr        column means
 #
 #
 #
 #

 getrmse <- function( title, nG, n, nr, B, Xr, Xl, W, rmsep ){
  # make B an array of n x n x nG
  dim(B)=c(n,n,nG)

  rmse=0;
  for( k in 1:nG ){
   TWT=Xl%*%Diag(W[k,])%*%t(Xr)   # W already squared
   rmse=rmse+sum((B[,,k]-TWT)^2)
  }
  rmse=sqrt(rmse/(nG*n*n))
  cat(title, ":", rmse, "\n")
  if( rmsep < rmse ){
   cat(title, ": **** failure !! ****", rmsep - rmse, " \n")
  }
  return( rmse )
 }


 # If Xall is given use it to calculate B matrix.
 Xgiven=0
 if( !is.null(Xall) ){
  Xgiven=1
  n=dim(Xall)[1]; nG=dim(Xall)[3]; nr=dim(Xall)[2]
  onen=matrix(1,n,1)
  meanX=matrix(0,nG,nr)
  if( estm == 1 ){
   for( k in 1:nG ){
    meanX[k,]=matrix(colMeans(Xall[,,k]),1,nr)
    Xall[,,k]=Xall[,,k]-onen%*%meanX[k,,drop=F]
   }
  }
  B=array(0,c(n,n,nG))
  for( k in 1:nG ){
   B[,,k]=Xall[,,k]%*%t(Xall[,,k])
  }
 }

 # const
 n=ncol(B);
 nG=dim(B)[3]

 if( print >= 1 ){
  cat("\n\nWeighted Euclidean MDS by INDSCAL", "\n")
  cat(" # of matrices = ", nG, ",  the size of X = ", n, " x ", nr, "\n")
  if( Xgiven ) cat(" X_k matrices will be converted to B_k matrices... \n")
  cat(" estimation of the column means = ", estm, "\n")
  cat(" max # of iterations = ", lmax, ",  eps = ", eps, "\n")
 }


 # initial value of X and W
 if( is.null(X) ){
 # sum(B) == Theta Lambda Theta'  (see Procrustes.tex)
 sumB=apply(B,1:2,sum)
 ev=eigen(sumB); eva=matrix(ev$values,length(ev$values),1); eve=ev$vectors;
 eva=eva[1:nr,]; eve=eve[,1:nr]
 Xc=eve%*%Diag(sqrt(eva))
 }
 else Xc=X
 Xl=Xc; Xr=Xc

 if( is.null(W) ) W=matrix(1,nG,nr)

 converged=0;
 rmse=0;
 for( k in 1:nG ){
  TWT=Xl%*%Diag(W[k,])%*%t(Xr)
  rmse=rmse+sum((B[,,k]-TWT)^2)
 }
 rmsep=sqrt(rmse/(nG*n*n))
 Xlp=Xl; Xrp=Xr; Wp=W

 rmsep=getrmse( "initial", nG, n, nr, B, Xr, Xl, W, rmsep )


 # main iteration

 for( llll in 1:lmax ){

  # estimate W:  This W is W^2
  for( k in 1:nG ){
   Wk=solve((t(Xc)%*%Xc) * (t(Xc)%*%Xc) ) %*%
    vecdiag(t(Xc)%*%B[,,k]%*%Xc)
   W[k,]=t(Wk)
  }

  # negative Ws set on the boundary
  W[which(W<0)]=0

  if( print >= 4 )
   rmse0=getrmse( "after W", nG, n, nr, B, Xr, Xl, W, rmsep )


  WW=W%x%matrix(1,n)
  # Print(nrow(WW),ncol(WW))

  # make t(B) a vertically stacked matrix of  nG*n x n
  dim(B)=c(n,n*nG)
  # Print(nrow(B),ncol(B))

  # estimate Xl
  XX=matrix(rep(t(Xr),nG),,nr,byrow=1)*WW    # This is nG*n x nr matrix
  # Print(Xr, matrix(rep(t(Xr),nG),,nr,byrow=1),nrow(XX),ncol(XX))
  Xl=t( solve(t(XX)%*%XX)%*%t(XX)%*%t(B) )
  if( print >= 4 )
   rmse0=getrmse( "after Xl", nG, n, nr, B, Xr, Xl, W, rmse0 )

  # normalize Xl so that diag(t(Xl)%*%Xl) = n I
  dd=diag( sqrt( n/diag(t(Xl)%*%Xl) ) )
  Xl=Xl%*%dd;
  W=W%*%solve(dd*dd);
  WW=W%x%matrix(1,n)
  if( print >= 4 )
   rmse0=getrmse( "after norm Xl", nG, n, nr, B, Xr, Xl, W, rmse0 )


  # estimate Xr
  XX=matrix(rep(t(Xl),nG),,nr,byrow=1)*WW    # This is nG*n x nr matrix
  Xr=t( solve(t(XX)%*%XX)%*%t(XX)%*%t(B) )
  if( print >= 4 )
   rmse0=getrmse( "after Xr", nG, n, nr, B, Xr, Xl, W, rmse0 )

  # normalize Xr so that diag(t(Xr)%*%Xr) = n I
  dd=diag( sqrt( n/diag(t(Xr)%*%Xr) ) )
  Xr=Xr%*%dd;
  W=W%*%solve(dd*dd);
  if( print >= 4 )
   rmse0=getrmse( "after norm xr", nG, n, nr, B, Xr, Xl, W, rmse0 )

  # mean
  Xc=(Xl+Xr)/2


  # make B an array of n x n x nG
  dim(B)=c(n,n,nG)

  rmse=0;
  for( k in 1:nG ){
   TWT=Xl%*%Diag(W[k,])%*%t(Xr)
   rmse=rmse+sum((B[,,k]-TWT)^2)
  }
  rmse=sqrt(rmse/(nG*n*n))
  rmseimp=rmsep-rmse;
  rmseimpr=rmseimp/(rmsep+1e-7);
  maxadXl=max(abs(Xlp-Xl));
  maxadXr=max(abs(Xrp-Xr));
  maxadW=max(abs(W-Wp));

  if( print >= 2 )
   Print(llll, rmsep, rmse, rmseimp, maxadXl, maxadXr, maxadW)

  if ( rmseimpr < eps || maxadW < eps) {
   converged=1;
   break;
  }

  # next iteration;
  Tp=T; Wp=W; Xlp=Xl; Xrp=Xr
  rmsep=rmse;


 } # end of llll loop


 if ( !converged )
  cat("\n Warning(indscal): Iteration did not converge.\n\n")


 # square root of W
 W[which(W<0)]=0
 W=sqrt(W)

 # normalize Xc so that diag(t(Xc)%*%Xc) = n I
 dd=diag( sqrt( n/diag(t(Xc)%*%Xc) ) )
 Xc=Xc%*%dd;
 W=W%*%solve(dd);

 # sort the dimensions according to W^2
 od=nr+1-order( colSums(W*W) )
 W=W[,od]
 Xc=Xc[,od]
 maxadXlr=max(abs(Xl-Xr))

 rmseB=0;
 for( k in 1:nG ){
  TWT=Xc%*%Diag(W[k,]^2)%*%t(Xc)
  rmseB=rmseB+sum((B[,,k]-TWT)^2)
 }
 rmseB=sqrt(rmseB/(nG*n*n))

 # find T such that X_k = Xc W_k T_k by orthogonal Procrustes
 T=array(0,c(nr,nr,nG))
 rmseX=NA
 if( Xgiven ){
  for( k in 1:nG ){
   # rotate Xc diag(W_k) to match X_k
   XcW=Xc%*%Diag(W[k,])
   CAAC=t(Xall[,,k])%*%XcW%*%t(XcW)%*%Xall[,,k];
   ev=eigen(CAAC); eva=matrix(ev$values,length(ev$values),1); eve=ev$vectors;
   rm(ev); loc=which(eva > 0); eva=eva[loc]; eve=eve[,loc];
   SQR=eve%*%Diag(sqrt(1/eva))%*%t(eve);
   Tk=t(XcW)%*%Xall[,,k]%*%SQR;
   T[,,k]=Tk
   if( estm == 1 ){
    Xall[,,k]=Xall[,,k]+onen%*%meanX[k,,drop=F]
   }
  }
  rmseX=0;
  for( k in 1:nG ){
   XWT=Xc%*%Diag(W[k,])%*%T[,,k] + onen%*%meanX[k,,drop=F]
   rmseX=rmseX+sum((Xall[,,k]-XWT)^2)
   if( print >= 3 ) Print(k,"/",Xall[,,k],XWT, digits=2)
  }
  rmseX=sqrt(rmseX/(nG*n*nr))
 } # end of Xgiven

 temp=mandd(Xc)
 mean=temp$mean
 Disp=temp$cov
 tWW=t(W)%*%W

 if( print >= 1 ){
  cat("\n\nWeighted Euclidean MDS by INDSCAL", "\n")
  cat("  # of matrices = ", nG, ",  the size of X = ", n, " x ", nr, "\n")
  if( Xgiven ) cat(" X_k matrices were converted to B_k matrices... \n")
  cat(" estimation of the column means = ", estm, "\n")
  cat(" # of iterations = ", llll, ",  eps = ", eps, "\n")
  cat(" rmse in terms of B = ", rmseB, ", rmse in terms of X = ", rmseX, "\n")
  cat(" max abs difference between Xl and Xr = ", maxadXlr, "\n")
  cat("\n Common Configuration Matrix and weight matrices: \n\n")
  Print(Xc, W, fmt="7.3")
  Print(mean,Disp, tWW, fmt="7.3")
  if( Xgiven ){
   if( estm == 1 ){
    cat(" column means of Xall \n"); print(meanX)
   }
   cat("\n Rotation matrix of the form:  X_k = X W_k T_k  \n\n")
   Print(T, fmt="7.3")
  }
 }


 res=list(X=Xc, W=W, T=T, rmseB=rmseB, rmseX=rmseX)

 return( res )

} # end of indscal



set.seed(1701)
# generate O from X and W
X <- matrix( c( 0,4, 0,0, 3,0, 3,4, 1.5,2 ), ,2, byrow=1 )
dimnames(X) <- list(paste("s",1:nrow(X),sep=""), paste("d",1:ncol(X),sep=""))
X <- standard( X, center=1 )
W <- matrix( c( 1,1, 2,1, 1,2 ),,2, byrow=1 )
rownames(W) <- paste("sbj",1:nrow(W),sep=""); colnames(W) <- colnames(X)
resgen <- gendatamds( X=X, W=W, stderror=0.05, print=2 )
O <- vechinv(resgen$vO, nodiag=1, array=1)  # This is n x n x nG.
# double center O
n <- nrow(X); nG <- nrow(W)
J <- diag(n)-(1/n)*matrix(1,n,1)%*%matrix(1,1,n)
B <- O
for( k in 1:nG ){
 B[,,k] <- -0.5*J%*%O[,,k]^2%*%J
}
Print(O,B, fmt=".2")


res2 <- sumscal( B, nr=2, estm=0, print=2 )
res1 <- indscal( B, nr=2, estm=0, print=2, X=res2$X, W=res2$W )


Xi=res1$X
Wi=res1$W
Ti=res1$T
mi=res1$m



comments('

seed <- 1701; n <- 20; nr <- 2; pmiss <- 0; errstd <- 0.05
set.seed(seed)
resg <- gendataWmm( 3, n, nr, genm=1, genW=2, pmiss=pmiss, errstd=errstd)
A <- resg$A; Xall <- l2a(resg$C); Tg <- resg$T; mg <- resg$m; Wg <- resg$W
res <- indscal( NULL, nr, estm=1, lmax=lmax, eps=eps, print=2, X=Xall )

')















