



#' Simultaneous Diagonalization of Several Symmetric Matrices
#'
#' @param C Input array of symmetric matrices
#' @param lmax max # of iterations
#' @param eps convergence criterion
#' @param print = 1 to print result
#'
#' @details
#'  Given matrices \eqn{C_k, n=1,2,...,nG}, find the orthonormal matrix T
#'  such that
#' \deqn{
#'      T' C_k T == W_k,    k=1,2,...,nG, diag
#'      }
#'    OR
#' \deqn{
#'      C_k == T W_k T',    k=1,2,...,nG
#'      }
#'  \cr
#'  This program minimizes
#' \deqn{
#'    rss2 = sum_k (  (C_k - T W_k T')^2 )
#'    }
#'  with respect to orthogonal T and diagonal W_k. \cr
#'
#' @return A list of \cr
#' T, W, rmse, llll=iteretions needed, converged=1 if converged.
#'
#'
#' @examples
#' set.seed(1701)
#' W <- matrix( c( 1,1, 2,1, 1,2 ),,2, byrow=1 )
#' rownames(W) <- paste("sbj",1:nrow(W),sep="")
#' colnames(W) <- paste("d",1:ncol(W),sep="")
#' theta <- pi/3
#' T <- matrix(c(cos(theta),sin(theta),-sin(theta),cos(theta)),2,2)
#' C <- array(0,dim=c(2,2,3))
#' for( k in 1:nrow(W) ){
#'  C[,,k] <- T%*%diag(W[k,])%*%t(T)
#'  E <- 0.05*matrix(rnorm(4),2,2);    E <- (E+t(E))/2
#'  C[,,k] <- C[,,k] + E
#' }
#' dimnames(C) <- list(colnames(W),colnames(W),rownames(W))
#'
#' res <- smscl( C, lmax=100, eps=1e-8, print=2 )
#'
#'
#'
#' @references
#' De Leeuw, Jan, and  Pruzansky, Sandra (1978)
#' A New Computational Method to Fit the Weighted Euclidean Distance Model.
#' Psychometrika, v43 n4 p479-9
#'
#' @export
#'
#'

smscl <- function( C, lmax=100, eps=1e-8, print=2 ){
 # Simultaneous Diagonalization of Several Symmetric Matrices
 #  Shin-ichi Mayekawa
 #  The original Fortran version is in Takane(1980)
 #  Basic version as a module for CADA-80: 1983
 #  SAS Matrix/IML version: : 1985-1993, 080327
 #  120107,08
 #  return llll and converged: 120109
 #
 # Given matrices C_k, n=1,2,...,nG, find the orthonormal matrix T
 # such that
 #     T' C_k T == W_k,    k=1,2,...,nG, diag
 #   OR
 #     C_k == T W_k T',    k=1,2,...,nG
 #
 # This program minimizes
 #   rss2 = sum_k (  (C_k - T W_k T')^2 )
 # with respect to orthogonal T and diagonal W.
 #
 # Args:
 #  C    nr x nr x nG  array to be diagonalized
 #

 # const
 nr=ncol(C)
 nG=dim(C)[3]
 Corg=C

 P4=.25*3.14159265359
 P8=.5*P4

 # initial
 T=diag(nr)
 W=matrix(0,nG,nr)
 for( k in 1:nG ){
  W[k,]=t(vecdiag(C[,,k]))
 }

 rmse=0; rmse1=0; rmse2=0
 for( k in 1:nG ){
  TWT=T%*%Diag(W[k,])%*%t(T)
  rmse=rmse+sum((C[,,k]-TWT)^2)
  rmse1=rmse1+sum( (C[,,k]-Diag(W[k,]))^2 )
  rmse2=rmse2+sum( (Corg[,,k]-TWT)^2 )
 }
 rmsep=sqrt(rmse/(nG*nr*nr))
 rmse1p=sqrt(rmse/(nG*nr*nr))
 rmse2p=sqrt(rmse/(nG*nr*nr))


 if( print >= 1 ){
  cat("\n\n smscl: Simultaneous Diagonalization of Several Symmetric Matrices",
      "\n")
  cat(" # of matrices = ", nG, ",  the size of each matrix = ", nr, "\n")
  cat(" max # of iterations = ", lmax, ",  eps = ", eps, "\n\n")
 }


 converged=0

 # main iteration
 for( llll in 1:lmax ){

  c9=1
  for( i in 2:nr ){
   for( j in 1:(i-1) ){

    d1=C[j,j,]-C[i,i,]
    d2=C[i,j,]

    aa=sum(d1*d1)/4
    b=sum(d2*d2)
    g=sum(d1*d2)/2
    d=aa-b

    # Determine T1 by checking the signs of d and g.
    if( d > 0  &&  abs(g) < eps ) next

    if( abs(d) < eps ){
     cat(" warning1:(sumscl) too small d ", d )
     t1=-P8*sign(g)
    }
    else{
     e=atan(-2*g/d)
     t1=.25*e
     if( d <= 0 ){
      if( g > 0 ) t1=t1-P4
      else        t1=t1+P4
     }
    }
    s1=sin(t1)
    c1=sqrt(1-s1*s1)
    if( c1 < c9 ) c9=c1

    if( print >= 4 ) Print( llll, i, j, d1, d2, aa, b, g, d, e, t1 )

    # update C and T matrices
    Tij=diag(nr)
    Tij[i,i]=c1; Tij[i,j]=-s1; Tij[j,i]=s1; Tij[j,j]=c1
    T=T%*%Tij

    for( k in 1:nG ){
     C[,,k]=t(Tij)%*%C[,,k]%*%Tij
    }

   } # end of j loop
  } # end of i loop

  # get W
  for( k in 1:nG ){
   W[k,]=t(vecdiag(C[,,k]))
  }

  # convergence
  rmse=0; rmse1=0; rmse2=0
  for( k in 1:nG ){
   TWT=T%*%Diag(W[k,])%*%t(T)
   rmse=rmse+sum((C[,,k]-TWT)^2)
   rmse1=rmse1+sum( (C[,,k]-Diag(W[k,]))^2 )
   rmse2=rmse2+sum( (Corg[,,k]-TWT)^2 )
  }
  rmse=sqrt(rmse/(nG*nr*nr))
  rmse1=sqrt(rmse1/(nG*nr*nr))
  rmse2=sqrt(rmse2/(nG*nr*nr))
  rmse2imp=(rmse2p-rmse2);

  if( print >= 2 )
   Print( llll, rmsep, rmse, rmse1, rmse2, rmse2imp, c9 )

  if( rmse2imp < eps ){
   converged=1
   break
  }

  # next iteration
  rmsep=rmse;
  rmse2p=rmse2;


 } # end of llll loop


 if( !converged )
  cat(" warning1:(smscl) iteration not converged.", rmse2imp, lmax, eps )

 # sort the dimensions according to W^2
 od=nr+1-order( colSums(W*W) )
 W[,od]=W
 T[,od]=T
 rmse=0;
 for( k in 1:nG ){
  TWT=T%*%Diag(W[k,])%*%t(T)
  rmse=rmse+sum((Corg[,,k]-TWT)^2)
 }
 rmse=sqrt(rmse/(nG*nr*nr))

 if( print >= 1 ){
  tTT=t(T)%*%T
  cat("\n\n smscl: Simultaneous Diagonalization of Several Symmetric Matrices",
      "\n")
  cat(" # of matrices = ", nG, ",  the size of each matrix = ", nr, "\n")
  cat(" max # of iterations = ", lmax, ",  eps = ", eps, "\n")
  cat(" Diagonalized matrices: \n\n")
  Print(C, fmt="8.4")
  cat(" Rotation matrix of the form:  T' C_k T = W_k  \n\n")
  Print(T, tTT, fmt="8.4")
 }

 res=list( T=T, W=W, rmse=rmse, llll=llll, converged=converged )
 return( res )

} # end of smscl






comments('

# Convert vertically stacked matrices (supermat) to array
# The following C matrix contains eight 4 x 4 symmetric matrices.
#  ( Line breaks are not needed. )
#
C="
  0.424   0.061   0.010  -0.176
  0.061   0.342   0.006  -0.049
  0.010   0.006   0.768   0.024
 -0.176  -0.049   0.024   0.552

  0.404   0.209   0.064  -0.328
  0.209   0.438   0.011   0.017
  0.064   0.011   0.148  -0.099
 -0.328   0.017  -0.099   0.692

  0.740   0.061   0.018  -0.064
  0.061   0.785   0.001   0.026
  0.018   0.001   0.577  -0.038
 -0.064   0.026  -0.038   0.808

  0.502   0.097   0.013   0.024
  0.097   0.725  -0.009   0.128
  0.013  -0.009   0.185  -0.053
  0.024   0.128  -0.053   0.551

  0.587  -0.243  -0.054   0.286
 -0.243   0.433  -0.003  -0.086
 -0.054  -0.003   0.764   0.085
  0.286  -0.086   0.085   0.293

  0.365   0.249   0.061  -0.363
  0.249   0.441   0.008   0.040
  0.061   0.008   0.292  -0.080
 -0.363   0.040  -0.080   0.700

  0.122   0.113   0.010  -0.045
  0.113   0.299  -0.007   0.101
  0.010  -0.007   0.079  -0.022
 -0.045   0.101  -0.022   0.219

  0.776  -0.186   0.007   0.075
 -0.186   0.486   0.017  -0.165
  0.007   0.017   0.361  -0.036
  0.075  -0.165  -0.036   0.609
";
C=t( cards( C, header=0 ) )
dim(C)=c(nrow(C),nrow(C),ncol(C)/nrow(C))
Print(C)



lmax=50; eps=1e-7;
print=2;
res=smscl( C, lmax, eps, print )


')



nG <- 4; n <- 9; nr <- 3
resg=gendataWmm( nG, n, nr, genT=1, genW=2, eqT=1, pcA=2, errstd=0.05)
Xall=l2a(resg$C)
C=array( apply( Xall,3,function(X){t(X)%*%X} ), dim=c(nr,nr,nG) )
res=smscl( C, lmax, eps, print=2 )







