#' SUMSCAL
#'
#' @param B Input scalar product array
#' @param nr # of dimensions
#' @param lmax max # of iterations
#' @param eps convergence criterion for B
#' @param print = 1 to print result
#' @param estm = 1 to subtract column mean from X when Xall is given.
#' @param Xall An Array of X_k's, or NULL

#'
#' @details
#' Let B_k, k=1,2, ... , nG, be the scalar product matrices calculated
#' from the distance matrices, D_k, as \cr
#' \deqn{
#'  B_k = -2 J D_k^2 J
#' }
#' where \eqn{J} is the column centering operator. \cr
#' This program find those Xc and diagonal W_k matrices that minimize
#' \deqn{
#'   RSS = tr( (B_k -  Xc diag(W_k)^2 Xc')' (B_k -  Xc diag(W_k)^2 Xc') ) .
#'   }
#' \cr
#'  Note that, if X_k can be expressed as\
#' \deqn{
#'   X_k = Xc diag(W_k) T_k
#'   }
#'  we have
#' \deqn{
#'   B_k == X_k X_k' = Xc diag(W_k)^2 Xc'
#'   }
#'  Therefore, given nG X_k matrices, we may first calculate B_k matrices,\cr
#'  find Xc and W_k matrices by SUMSCAL, and then find T_k matrices.\cr
#' \cr
#' When X is present, it has priority over B. \cr
#' \cr
#'  Xc matrix will be normalized so that diag(t(Xc)%*%Xc) = n I .\cr
#'
#' @return A list of \cr
#' X=Xc, W, T, rmseB, rmseX
#'
#' @examples
#' set.seed(1701)
#' # generate O from X and W
#' X <- matrix( c( 0,4, 0,0, 3,0, 3,4, 1.5,2 ), ,2, byrow=1 )
#' dimnames(X) <- list(paste("s",1:nrow(X),sep=""), paste("d",1:ncol(X),sep=""))
#' X <- standard( X, center=1 )
#' W <- matrix( c( 1,1, 2,1, 1,2 ),,2, byrow=1 )
#' rownames(W) <- paste("sbj",1:nrow(W),sep=""); colnames(W) <- colnames(X)
#' resgen <- gendatamds( X=X, W=W, stderror=0.05, print=2 )
#' O <- vechinv(resgen$vO, nodiag=1, array=1)  # This is n x n x nG.
#' # double center O
#' n <- nrow(X); nG <- nrow(W)
#' J <- diag(n)-(1/n)*matrix(1,n,1)%*%matrix(1,1,n)
#' B <- O
#' for( k in 1:nG ){
#'  B[,,k] <- -0.5*J%*%O[,,k]^2%*%J
#' }
#' Print(O,B, fmt=".2")
#'
#' res1 <- indscal( B, nr=2, print=1 )
#' res2 <- sumscal( B, nr=2, print=1 )
#'
#'
#' @references
#' De Leeuw, Jan, and  Pruzansky, Sandra (1978)
#' A New Computational Method to Fit the Weighted Euclidean Distance Model.
#' Psychometrika, v43 n4 p479-9
#'
#' @export
#'

sumscal <- function( B, nr=2, lmax=50, eps=1e-7, print=2, estm=0, Xall=NULL ){
 # Sumscal
 # Shin-ichi Mayekawa
 # 120107,08,09
 # estm : 120116
 # bugfix for meanX: 20151127
 #
 # Given nG B_k matrices, this program finds Xc and W_k such that
 #      B_k == Xc diag(W_k)^2 Xc'
 #  Xc and W_k matrices are estimated by SUMSCAL algorithm
 #  assuming sum_k W_k^2 = I
 #
 #  Note that
 #   B_k == X_k X_k'   where X_k = Xc diag(W_k) T_k
 #  Therefore, given nG X_k matrices, we may first calculate B_k matrices,
 #  find Xc and W_k matrices, and then find T_k matrices.
 #
 #  Xc matrix will be normalized so that diag(t(Xc)%*%Xc) = n I
 #    ( NOT   sum_k W_k = I  )
 #  Dimensions will be ordered according to W[##,]
 #
 # Args:
 #   B     n x n x nG array of scalar products
 #
 #   Xall     n x nr x nG array of scale values
 #         If Xall is given, B will be calculated from Xall.
 #
 #   estm  = 1 to estimate the column menas when Xall is given
 #
 # Value: as list
 #
 #    Xc   n x nr
 #    W    nG x nr        containing the diagonal elements of W_k
 #    T    nr x nr x nG   array of T_k matrices
 #    m    nG x nr        column means
 #
 #
 # Needs:
 #  smscl.R
 #
 #

 # If Xall is given use it to calculate B matrix.
 Xgiven=0
 meanX=matrix(0,nG,nr)
 if( !is.null(Xall) ){
  Xgiven=1
  n=dim(Xall)[1]; nG=dim(Xall)[3]; nr=dim(Xall)[2]
  onen=matrix(1,n,1)
  if( estm == 1 ){
   for( k in 1:nG ){
    meanX[k,]=matrix(colMeans(Xall[,,k]),1,nr)
    Xall[,,k]=Xall[,,k]-onen%*%meanX[k,,drop=F]
   }
  }
  B=array(0,c(n,n,nG))
  for( k in 1:nG ){
   B[,,k]=Xall[,,k]%*%t(Xall[,,k])
  }
 }


 # const
 n=ncol(B);
 nG=dim(B)[3]

 if( print >= 1 ){
  cat("\n\nWeighted Euclidean MDS by SUMSCAL", "\n")
  cat(" # of matrices = ", nG, ",  the size of X = ", n, " x ", nr, "\n")
  if( Xgiven ) cat(" X_k matrices will be converted to B_k matrices... \n")
  cat(" estimation of the column means = ", estm, "\n")
  cat(" max # of iterations = ", lmax, ",  eps = ", eps, "\n")
 }


 # sum(B) == Theta Lambda Theta'  (see Procrustes.tex)
 sumB=apply(B,1:2,sum)
 ev=eigen(sumB); eva=matrix(ev$values,length(ev$values),1); eve=ev$vectors;
 eva=eva[1:nr,]; eve=eve[,1:nr]
 # Theta Lambda^(-1/2)
 ThetaLambdaMH=eve%*%Diag(1/sqrt(eva))

 # C_k = Lambda^(-1/2) Theta' B_k Theta Lambda^(-1/2)
 C=array(0,c(nr,nr,nG))
 for( k in 1:nG ){
  C[,,k]=t(ThetaLambdaMH)%*%B[,,k]%*%ThetaLambdaMH;
 }

 # use smscl to find S and W:  This is the only iterative part.
 res=smscl( C, lmax, eps, print-4 )
 rm(C)
 S=res$T
 W=res$W
 llll=res$llll

 # Xc = Theta Lambda^(1/2)
 Xc=eve%*%Diag(sqrt(eva))%*%S

 if( print >= 4 ){
  rmse=0;
  for( k in 1:nG ){
   TWT=Xc%*%Diag(W[k,])%*%t(Xc)
   rmse=rmse+sum((B[,,k]-TWT)^2)
  }
  rmse=sqrt(rmse/(nG*n*n))
  Print("after smscl", rmse)
 }

 # square root of W
 # negative Ws set on the boundary
 W[which(W<0)]=0
 W=sqrt(W)

 if( print >= 4 ){
  rmse=0;
  for( k in 1:nG ){
   TWT=Xc%*%Diag(W[k,]^2)%*%t(Xc)
   rmse=rmse+sum((B[,,k]-TWT)^2)
  }
  rmse=sqrt(rmse/(nG*n*n))
  Print("after sqrtW", rmse)
 }

 # normalize Xc so that diag(t(Xc)%*%Xc) = n I
 dd=diag( sqrt( n/diag(t(Xc)%*%Xc) ) )
 Xc=Xc%*%dd;
 W=W%*%solve(dd);

 if( print >= 4 ){
  rmse=0;
  for( k in 1:nG ){
   TWT=Xc%*%Diag(W[k,]^2)%*%t(Xc)
   rmse=rmse+sum((B[,,k]-TWT)^2)
  }
  rmse=sqrt(rmse/(nG*n*n))
  Print("after norm Xc", rmse)
 }

 # sort the dimensions according to W^2
 od=nr+1-order( colSums(W*W) )
 W=W[,od]
 Xc=Xc[,od]

 rmseB=0;
 for( k in 1:nG ){
  TWT=Xc%*%Diag(W[k,]^2)%*%t(Xc)
  rmseB=rmseB+sum((B[,,k]-TWT)^2)
 }
 rmseB=sqrt(rmseB/(nG*n*n))


 # find T such that X_k = Xc W_k T_k by orthogonal Procrustes
 T=array(0,c(nr,nr,nG))
 rmseX=NA
 if( Xgiven ){
  for( k in 1:nG ){
   # rotate Xc diag(W_k) to match X_k
   XcW=Xc%*%Diag(W[k,])
   CAAC=t(Xall[,,k])%*%XcW%*%t(XcW)%*%Xall[,,k];
   ev=eigen(CAAC); eva=matrix(ev$values,length(ev$values),1); eve=ev$vectors;
   rm(ev); loc=which(eva > 0); eva=eva[loc]; eve=eve[,loc];
   SQR=eve%*%Diag(sqrt(1/eva))%*%t(eve);
   Tk=t(XcW)%*%Xall[,,k]%*%SQR;
   T[,,k]=Tk
   if( estm == 1 ){
    Xall[,,k]=Xall[,,k]+onen%*%meanX[k,,drop=F]
   }
  }
  rmseX=0;
  for( k in 1:nG ){
   XWT=Xc%*%Diag(W[k,])%*%T[,,k] + onen%*%meanX[k,,drop=F]
   rmseX=rmseX+sum((Xall[,,k]-XWT)^2)
   if( print >= 3 ) Print(k,"/",Xall[,,k],XWT, digits=2)
  }
  rmseX=sqrt(rmseX/(nG*n*nr))
 } # endof Xgiven

 temp=mandd(Xc)
 mean=temp$mean
 Disp=temp$cov
 tWW=t(W)%*%W

 if( print >= 1 ){
  cat("\n\nWeighted Euclidean MDS by SUMSCAL", "\n")
  cat(" # of matrices = ", nG, ",  the size of X = ", n, " x ", nr, "\n")
  if( Xgiven ) cat(" X_k matrices were converted to B_k matrices... \n")
  cat(" estimation of the column means = ", estm, "\n")
  cat(" # of iterations = ", llll, ",  eps = ", eps, "\n")
  cat(" rmse in terms of B = ", rmseB, " rmse in terms of X = ", rmseX, "\n\n")
  cat("\n Common Configuration Matrix and weight matrices: \n\n")
  Print(Xc, W, fmt="7.3")
  Print(mean,Disp, tWW, fmt="7.3")
  if( Xgiven ){
   if( estm == 1 ){
    cat(" column means of Xall \n"); print(meanX)
   }
   cat("\n Rotation matrix of the form:  X_k = X W_k T_k  \n\n")
   Print(T, fmt="7.3")
  }
 }


 res=list(X=Xc, W=W, T=T, m=meanX, rmseB=rmseB, rmseX=rmseX)

 return( res )

} # end of sumscal





set.seed(1701)
# generate O from X and W
X <- matrix( c( 0,4, 0,0, 3,0, 3,4, 1.5,2 ), ,2, byrow=1 )
dimnames(X) <- list(paste("s",1:nrow(X),sep=""), paste("d",1:ncol(X),sep=""))
X <- standard( X, center=1 )
W <- matrix( c( 1,1, 2,1, 1,2 ),,2, byrow=1 )
rownames(W) <- paste("sbj",1:nrow(W),sep=""); colnames(W) <- colnames(X)
resgen <- gendatamds( X=X, W=W, stderror=0.05, print=2 )
O <- vechinv(resgen$vO, nodiag=1, array=1)  # This is n x n x nG.
# double center O
n <- nrow(X); nG <- nrow(W)
J <- diag(n)-(1/n)*matrix(1,n,1)%*%matrix(1,1,n)
B <- O
for( k in 1:nG ){
 B[,,k] <- -0.5*J%*%O[,,k]^2%*%J
}
Print(O,B, fmt=".2")


res2 <- sumscal( B, nr=2, estm=0, print=9 )







comments('


seed=213409875
set.seed(seed)


nr=3;
lmax=500; eps=1e-8
print=2


nG=3; n=9; nr=2
errstd=.1
resg=gendataWmm( nG, n, nr, genW=2, pcA=1, eqW=0, errstd=errstd )
Call=resg$C
Xall=array(unlist(Call), dim = c(dim(Call[[1]]), length(Call)))
rm(Call)
Xg=resg$A; Tg=resg$T; Wg=resg$W


res=sumscal( B, nr, estm=1, lmax=50, eps=1e-8, print=1, X=Xall )
Xs=res$X
Ws=res$W
Ts=res$T
ms=res$m


')


