#' Individual Differences Multidimensional Scaling
#' by the Gauss-Newton Method
#'
#'
#' @param O  n x n x nG array of observed dissimilarity matrices
#' @param ndim # of dimensions
#' @param init = 0 to use random initial for X matrix
#' @param mlevel = 1|2|3|4|5|6|9 \cr
#'   1="continuous nominal", 2="discrete nominal"
#' , 3="continuous ordinal(primary)", 4="discrete ordinal(secondary)"
#' , 5="interval", 6="monotone spline", 9="no transformation at all"
#' @param nknots # of knots for monotone spline transformation
#' @param estX = 0 to skip the estimation of X
#' @param estW = 0 to skip the estimation of W
#' @param X The initial value of X when estX=0
#' @param W The initial value of W when estW=0
#' @param SQUAREM = 3 to use SQUAREM method SqS3
#' @param nSQUAREM >= 1 # of iterations a which SQUAREM begins.
#' @param minalpha The minimul value of alpha parameter of SQUAREM
#' @param maxalpha <= 1 The maximum value of alpha parameter of SQUAREM
#' @param maxiter The maximum # of iterations
#' @param eps The convergence criterion for the relative improvement of rmse.
#' @param epsd The convergence criterion for the difference of param values.
#' @param print = 1 to print the result
#' @param debug = 1 to print the intermediate results
#'
#'
#' @details
#'
#' The puropose of this function is to demonstrate how
#' the ALS (Alternating Least Squares), the Gauss-Newton method,
#' and inline SQUAREM can be utilized in MDS context. \cr
#' The program may not be very efficient but it reduces rmse (stress2) at each
#' phase of iteration. \cr
#' \cr
#' The basic algorithm is as follows: \cr
#' 0) find initial value of X and W.\cr
#' 1) minimize rmse w.r.t. X given W by the GN method. \cr
#' 2) minimize rmse w.r.t. W given X by the GN method. \cr
#' 3) for every three iterations,
#' update X and W with inline SQUAREM if requested. \cr
#' 4) minimize rmse w.r.t. optimal transformation of O. \cr
#' 5) check convergence. \cr
#' 6) go back to step 1) if not converged. \cr
#' \cr\cr
#'
#'   This program minimizes the stress formula 2 in terms of the distance:
#'  \deqn{
#'  \sum_k \sum_{i>j} ( dh_{ijk} - d_{ijk} )^2 / variance( dh_{,,k} )
#'  }
#'    with respect to  X, W  and dh. \cr
#' \cr
#'   Actual minimization is done by minimizing
#'   \deqn{
#'       \sum_k \sum_{i>j} ( dh_{ijk} - d_{ijk} )^2
#'       }
#'  with respec to  X, W  and dh and normalizing each dh[,,k] so that its \cr
#'  mean is equal to the mean of d[,,k] and its varaiance is equal to unity.\cr
#' \cr
#'   Given X and W, the optimal dh[,,k] is found by regressing d[,,k] on o[,,k]
#'   under the suitable assumption specified by mlevel. \cr
#' \cr
#'   Given dh and W, X is updated using the Gauss-Newton method. \cr
#'   Given dh and X, W is updated using the Gauss-Newton method. \cr
#' \cr
#' \cr
#'   When nG >= 2, \cr
#'   X is normalized so that 1'X = 0' and diag(X'X) = n I. \cr
#'
#'   When nG = 1, W is set equal to 1 and 1'X = 0' will be enforced. \cr
#'
#'   When mlevel = 9, dh is set equal to o and will not be normalized.
#'
#'
#' \cr\cr\cr
#' For the monotone least squares transformation, see the description of
#' lazy.stat::mlsreg. \cr
#' For the monotone spline, see the description of lazy.stat::spreg
#' with type="m" option.
#'
#'
#' @return
#' A list of the following: \cr
#'  rmse The rmse minimized \cr
#'  stress1 stress1 \cr
#'  stress2 stress2 minimized \cr
#'  stress1_s stress1 for each individual \cr
#'  stress2_s stress2 for each individual \cr
#'  X n x ndim group configuration matrix \cr
#'  W nG x ndim subject weight matrix \cr
#'  vO The roll-out of the observed dissimilarity matrices.\cr
#'  vDhat The roll-out of the optimal transformation matrices.\cr
#'  vD The roll-out of the distance matrices.\cr
#'  iter_hist The history of the iteration with the columns: \cr
#'   llll, rmse, maxadX, maxadW. \cr
#'  mlevel, nknots, ndim
#'
#'
#'
#' @examples
#' # generate data
#' set.seed(1701)
#' n=10; nG=3; ndim=2
#' npair=n*(n-1)/2
#' X=matrix(rnorm(n*ndim), n, ndim)
#' W=matrix(runif(nG*ndim), nG, ndim)
#' W=W%*%Diag(1/sqrt(colSums(W^2)))
#' # weighted Euclidean distance
#' vD=vech(distancew( X, W ),nodiag=1)
#' # add scaled error
#' stdD=sqrt( diag(cov(vD)*(npair-1)/npair) )
#' stderr=.2
#' vO=vD + matrix(rnorm(npair*nG),npair,nG )%*%Diag(stderr*stdD)
#' vO[vO<0]=0
#' # back to square array
#' O=vechinv(vO, nodiag=1, array=1) # this is n x n x nG
#' res1=wmdsGN( O, ndim=2 , print=2 )
#' plot_wmds( res1 )
#' # res1=wmdsGN( O, ndim=2 , print=2, SQUAREM=0 )
#'
#'
#' @export
#'

wmdsGN <- function( O, ndim, init=1, mlevel=5, nknots=5
                    , estX=1, estW=1, X=NULL, W=NULL
                    , SQUAREM=3, nSQUAREM=3, minalpha=-9999, maxalpha=-1
                    , maxiter=100, eps=1e-6, epsd=1e-6, print=2, debug=0 ){
 # weighted mds by the Gauss Newton method
 # Shin-ichi Mayekawa
 # 130109
 # def of vX is transposed to vX = vec(t(X)) = matrix(t(X),,1):130113
 # SQUREM: 130114,15
 # normalization bugfix: 130115
 # matrix as array when nG=1: 130115
 # Diag when nG=1: 130116
 # use of lazy.mat: vech, vechinv and JacobianMat: 20151124
 # bugfix for mlevel=5 (new mlsreg): 20151126
 # SQUAREM simplified: 20151126
 # output vO: 20151127
 # bugfix for JacobianMat: 20151219
 # normX=0 if estW=0: 20151219
 #
 #
 # Args:
 #
 #  O       n x n x nG    3d array
 #          or   n x n matrix
 #
 #  X, W    initial config or NULL
 #  init    = 1 to use cmds of the average O when X is NULL
 #
 #  mlevel = 1, 2, 3, 4, 5,  or 9( no trans formation )
 #  Cannot use mlevel=6 because we use var(dh) to normalize dh.
 #
 #
 # Note:
 #  This program minimizes
 #     \sum_k \sum_{i>j} ( dh_{ijk} - d_{ijk} )^2 / variance( dh_{,,k} )
 #   with respect to  X, W  and dh.
 #
 #  Actual minimization is done by minimizing
 #      \sum_k \sum_{i>j} ( dh_{ijk} - d_{ijk} )^2
 #   with respec to  X, W  and dh and normalizing each dh[,,k] so that its
 #   mean is equal to the mean of d[,,k] and its varaiance is equal to unity.
 #
 #  Given X and W, the optimal dh[,,k] is found by regressing d[,,k] on o[,,k]
 #  under the suitable assumption specified by mlevel.
 #
 #  Given dh and W, X is updated using the Gauss-Newton method.
 #  Given dh and X, W is updated using the Gauss-Newton method.
 #
 #  When nG >= 2,
 #  X is normalized so that 1'X = 0' and diag(X'X) = n I.
 #
 #  When nG = 1, W is set equal to 1' and 1'X = 0' will be enforced.
 #
 #  When mlevel = 9, dh is set equal to o and will not be normalized.
 #


 get_rmse <- function( vO, vX, vW ){
  # rmse defined in terms of the lower-half of O and D matrices
  vD=vech( distancew( t(matrix(vX,ndim)), t(matrix(vW,ndim)) ), nodiag=1 )
  rmse=sqrt( sum((vO-vD)^2)/length(vO) )
  return( list(rmse=rmse, vD=vD) )
 }# end of get_rmse


 distX <- function( vX ){
  # returns the lower half off diag of the distance matrix as a matrix
  # to be called from JacobianMat function
  # ndim and W comes from the parent frame.
  # Print("in dist:", ndim, W)
  return( vech( distancew( t(matrix(vX,ndim)), W ), nodiag=1 ) )
 }


 distW <- function( vW ){
  # returns the lower half off diag of the distance matrix as a matrix
  # to be called from JacobianMat function
  # ndim and X comes from the parent frame.
  # Print("in dist:", ndim, X)
  return( vech( distancew( X, t(matrix(vW,ndim)) ), nodiag=1 ) )
 }


 # const
 n=nrow(O)
 if( is.matrix(O) ) O=array(O,dim=c(n,n,1))
 nG=dim(O)[3]

 # do not normalize X when there are only one group/subject
 normX=1
 if( nG == 1  || estW == 0 ){
  estW=0
  normX=0
 }

 npair=n*(n-1)/2
 nX=n*ndim
 nW=nG*ndim
 mlev=c( "continuous nominal", "discrete nominal", "continuous ordinal(primary"
         , "discrete ordinal(secondary)", "interval", "monotone spline")

 # column wise roll-out of lower half off diagolan elements of O matrix
 vO=vech(O,nodiag=1) # this is npair x nG
 Dhat=O
 vDhat=vech(Dhat,nodiag=1) # this is npair x nG

 # initial X
 if( is.null(X) ){
  if( init == 1 ){
   # X=cmds( apply(Dhat,1:2,mean), ndim )
   X=cmdscale( apply(Dhat,1:2,mean), ndim )
   if( normX == 1 ) X=standard(X)
  }
  else
   X=matrix(rnorm(n*ndim),n)
 }
 # initial W
 if( is.null(W) ){
  W=matrix(1,nG,ndim)
 }

 # initial distance and its roll-out
 vX=matrix(t(X),,1)
 vW=matrix(t(W),,1)
 D=distancew( X, W )
 vD=vech( D, nodiag=1 )

 # initial Dhat
 if( mlevel <= 6 ){
  # update Dhat
  for( k in 1:nG )
   vDhat[,k]=mlsreg( vD[,k], vO[,k], method=mlevel )

  # normalize Dhat to unit variance while keeping the original mean
  vDhatmean=colMeans(vDhat)
  vDhat=standard(vDhat, center=1)
  temp=sqrt(colSums(vDhat^2)/npair)
  vDhat=vDhat%*%Diag(1/temp)+t(matrix(vDhatmean,nG,npair))
  Dhat=vechinv(vDhat, nodiag=1, array=1) # this is n x n x nG
  if( normX == 1 ){
   W=Diag(1/temp^2)%*%W
   vW=matrix(t(W),,1)
  }
 }
 temp=get_rmse( vDhat, vX, vW )
 rmse=temp$rmse
 vD=temp$vD

 if( print >= 1 ){
  cat("\n\nIndividual Differences Multidimensional Scaling\n\n")
  cat("\n # of objects    =", n, "\n")
  cat(" # of subjects   =", nG, "\n")
  cat(" # of dimensions =", ndim, "\n")
  cat("\n measurement level =", mlevel, "(", mlev[mlevel], ")\n")
  if( mlevel == 6 ) cat("  with ",nknots," equally spaced knots. \n")
  cat("\n estimation of X =", estX, ",  estimation of W =", estW, "\n")
  cat("\n max # of iterations =", maxiter,"\n")
  cat(" eps for the improvement of rmse =", eps,"\n")
  cat(" eps for the max. abs. diff. of X and W =", epsd,"\n")
  cat("\n SQUAREM =", SQUAREM, "\n")
  cat(" nSQUARE =", nSQUAREM, ",  minalpha =", minalpha
      , ",  maxalpha =", maxalpha, "\n")
  if( print >= 3)  Print("Initial Config\n", X, W, rmse, fmt="8.3 8.3 .6")
 }

 # previous values
 vXp=vX
 vWp=vW
 rmsep=rmse


 alpha=NA
 #############################################################(1)
 if( SQUAREM ){
  # SQUAREM
  if( estW == 1 ){
   paramnum_save=matrix(NA,((n+nG)*ndim),3)
   paramnum_save[,1]=c(vX,vW)
  }
  else{
   paramnum_save=matrix(NA,(n*ndim),3)
   paramnum_save[,1]=c(vX)
  }
  paramnum_save[,2]=paramnum_save[,1]
  paramnum_save[,3]=paramnum_save[,1]
  llll2=0
  nSQEM=0
  nSQEMs=0
 }
 #############################################################(1)


 iter_hist=matrix(c(0,rmse,0,0,alpha),1,5)
 colnames(iter_hist)=c("llll","rmse","maxadX","maxadW","alpha")


 # main iteration loop
 for( llll in 1:maxiter ){

  if( estX == 1 ){

   # update X by GN

   # Jacobian:  d vec(D) / d vec(X)   npair x n*ndim
   Jac=JacobianMat( vX, func=distX, eps = 1e-06 )

   # gradient and info mat and GN direction
   g=-t( t(matrix(vDhat-vD,,1))%*%Jac )
   H=t(Jac)%*%Jac
   d=Ginv(H, MASS=1)%*%g

Print(vX,g,d,W)

   # update X along the direction d
   ok=0
   step=1
   for( lllll in 1:10 ){
    vX1=vX-step*d
    temp=get_rmse( vDhat, vX1, vW )
    rmse1=temp$rmse
    if( rmse1 <= rmse ){
     vX=vX1
     vD=temp$vD
     rmse=rmse1
     ok=1
     break
    }
    step=step/2
   }
   if( ok == 0 ){
    Print("Big Trouble! X:\n", rmse, rmse1, fmt="12.7")
    # Print(X,W,rmse1)
   }

   if( debug ) Print("after X: ", rmse, fmt=".9")

   # center X
   X=t(matrix(vX,ndim))
   X=t( t(X)-colMeans(X) )
   vX=matrix(t(X),,1)

   if( normX == 1 ){
    # normalize X:
    X=t(matrix(vX,ndim))
    temp1=diag(cov(X)*(n-1)/n)
    #   X=standard(X)
    X=t( t(X)-colMeans(X) )%*%Diag(1/sqrt(temp1))
    vX=matrix(t(X),,1)
    W=W%*%Diag(temp1)
    vW=matrix(t(W),,1)
    rmse_0=get_rmse( vDhat, vX, vW )$rmse
    #Print("after X norm: ", rmse_0)
   } # end of norm X

  } # end of estX

Print("before W", vX, vW, vD, vDhat)

  if( estW == 1 ){

   # update W by GN

   # Jacobian:  d vec(D) / d vec(X)   npair x n*ndim
   Jac=JacobianMat( vW, func=distW, eps = 1e-06 )

   # Print( Jac, fmt="6.2")

   # gradient and info mat and GN direction
   g=-t( t(matrix(vDhat-vD,,1))%*%Jac )
   H=t(Jac)%*%Jac
   d=Ginv(H, MASS=1)%*%g

   # Print( g,d, "\n", H )

   # update W along the direction d
   ok=0
   step=1
   for( lllll in 1:10 ){
    vW1=vW-step*d
    vW1[vW1<1e-5]=1e-5
    temp=get_rmse( vDhat, vX, vW1 )
    rmse1=temp$rmse
    if( rmse1 <= rmse ){
     vW=vW1
     W=t(matrix(vW,ndim))
     vD=temp$vD
     rmse=rmse1
     ok=1
     break
    }
    step=step/2
   }
   if( ok == 0 ){
    Print("Big Trouble! W:\n", rmse, rmse1, fmt="12.7")
    # Print(X,W, rmse1)
   }

   if( debug ) Print("after W: ", rmse, fmt=".9")

  } # end of estW


  ###################################################################(2)
  # inline SQUAREM
  if( llll >= nSQUAREM  &&  SQUAREM ){

   # save history
   #
   llll2=llll2+1
   paramnum_save[,3]=paramnum_save[,2]          # 3 = theta_n (oldest)
   paramnum_save[,2]=paramnum_save[,1]          # 2 = F(theta_n)
   if( estW == 1 ) paramnum_save[,1]=c(vX,vW)   # 1 = F(F(theta_n))
   else paramnum_save[,1]=c(vX)

   # update every three iterationss
   if( llll2 %% 3 == 0 ){

    # calculate alpha
    nSQEM=nSQEM+1
    r=paramnum_save[,2] - paramnum_save[,3]
    v=paramnum_save[,1] - 2*paramnum_save[,2] + paramnum_save[,3]
    if( SQUAREM == 3 ) alpha=-sqrt(sum(r*r)/sum(v*v))
    else if( SQUAREM == 1 ) alpha=sum(v*r)/sum(v*v)

    if( alpha < minalpha ) alpha=minalpha
    if( alpha > maxalpha ) alpha=maxalpha

    # update parameter
    newp=paramnum_save[,3] -2*alpha*r + (alpha^2)*v
    vX0=newp[1:nX]
    if( estW == 1 ){
     vW0=newp[(nX+1):length(newp)]
     vW0[vW0<1e-5]=1e-5
    }
    else vW0=vW

    # evaluate rmse
    temp=get_rmse( vDhat, vX0, vW0 )
    rmse0=temp$rmse
    vD0=temp$vD

    # check if improved
    if( minalpha <= -9999  |  rmse0 <= rmse ){

     # improved
     vX=vX0
     vW=vW0
     X=t(matrix(vX,ndim))
     W=t(matrix(vW,ndim))

     # center X
     X=t( t(X)-colMeans(X) )
     vX=matrix(t(X),,1)

     if( normX == 1 ){
      # normalize X:
      cmX=colMeans(X)
      temp1=Diag((cov(X)*(n-1)/n))
      X=standard(X)
      vX=matrix(t(X),,1)
      W=W%*%temp1
      vW=matrix(t(W),,1)
     } # end of norm X

     vD=vD0
     rmse=rmse0

     # store the updated result as #1
     if( estW == 1 ) paramnum_save[,1]=c(vX, vW)
     else paramnum_save[,1]=c(vX)

     llll2=0   # counting next update as the first one.
     llll2=1   # counting this update as the first one.
     nSQEMs=nSQEMs+1

     if( print >= 2 ){
      cat("SQUAREM success ")
      Print( rmse, maxadX,maxadW, alpha, fmt=c(".7"))
     }

    } # end of success
    else{

     # not improved: do nothing
     llll2=1  #
     llll2=2  # persistent setting

    } # failure

   } # end of mod=3

  } # end of SQUAREM
  ###################################################################(2)

Print("after SQUAREM", vX,vW)
  # optimal transformation of O
  if( mlevel <= 6 ){

   # update Dhat:  regress vD on vOn by monotore LS regression
   if( mlevel <= 4 ){
    for( k in 1:nG )
     vDhat[,k]=mlsreg( vD[,k], vO[,k], method=mlevel )
   }
   else if( mlevel == 5 ){
    # regress vD on vOn by linear regression
    for( k in 1:nG ){
     b0=cov(vD[,k], vO[,k])/var(vO[,k])
     vDhat[,k]=b0*(vO[,k]-mean(vO[,k]))+mean(vD[,k])
    }
   }
   else{
    # monotone spline
    for( k in 1:nG ){
     vDhat[,k]=spreg( vD[,k], vO[,k], n=nknots, type="m")$yhat
    }
   }

   if( debug ){
    temp=get_rmse( vDhat, vX, vW )
    rmse=temp$rmse
    Print("after mlsreg:", rmse)
   }

   # normalize Dhat to unit variance while keeping the original mean
   # This is for the stress formula in which
   # the variance of Dhat is in the denominator.
   vDhatmean=colMeans(vDhat)
   #  vDmean=colMeans(vD)
   vDhat=standard(vDhat, center=1)
   temp=sqrt(colSums(vDhat^2)/npair)
   vDhat=vDhat%*%Diag(1/temp)+t(matrix(vDhatmean,nG,npair))
   Dhat=vechinv(vDhat, nodiag=1, array=1) # this is n x n x nG

   if( debug ){
    temp=get_rmse( vDhat, vX, vW )
    rmse=temp$rmse
    Print("after mlsreg normalization:", rmse)
   }

  } # end of optimal transformation


  # check convergence
  temp=get_rmse( vDhat, vX, vW )
  rmse=temp$rmse
  vD=temp$vD
  rmseimpr=(rmsep-rmse)/rmsep
  maxadX=max(abs(vX-vXp))
  maxadW=max(abs(vW-vWp))
  if( print >= 2 )
   Print( llll, lllll, rmse, rmsep, rmseimpr, maxadX, maxadW, alpha
          , fmt="i3 i2 .6")
  iter_hist=rbind( iter_hist, c(llll,rmse,maxadX, maxadX, alpha) )

  if( rmseimpr <= eps  &&  maxadX <= epsd ) break

Print(vD,vDhat)
  # next iteration
  rmsep=rmse
  vXp=vX
  vWp=vW


 } # end of llll loop


 niter=llll

 if( normX == 1 ){
  # normalize X:
  X=t(matrix(vX,ndim))
  temp=sqrt(Diag(cov(X))*(n-1)/n)
  X=standard(X)
  vX=matrix(t(X),,1)
  W=W%*%temp
  vW=matrix(t(W),,1)
 } # end of normX
 stdX=sqrt(diag(var(X)*(n-1)/n))
 corrX=cor(X)
 meanX=colMeans(X)


 # stress
 varvDhat=mandd(vDhat)$var*npair
 ssvDhat=apply(vDhat^2,2,sum)
 rss=apply((vD-vDhat)^2,2,sum)
 stress1=sqrt(sum(rss)/sum(ssvDhat))
 stress2=sqrt(sum(rss)/sum(varvDhat))
 stress1_s=sqrt(rss/ssvDhat)
 stress2_s=sqrt(rss/varvDhat)


 if( print >= 1 ){
  cat("\n\nIndividual Differences Multidimensional Scaling\n\n")
  cat("\n # of objects    =", n, "\n")
  cat(" # of subjects   =", nG, "\n")
  cat(" # of dimensions =", ndim, "\n")
  cat("\n measurement level =", mlevel, "(", mlev[mlevel], ")\n")
  if( mlevel == 6 ) cat("  with ",nknots," equally spaced knots. \n")
  cat("\n estimation of X =", estX, ",  estimation of W =", estW, "\n")
  cat("\n max # of iterations =", maxiter,"\n")
  cat(" eps for the improvement of rmse =", eps,"\n")
  cat(" eps for the max. abs. diff. of X and W =", epsd,"\n")
  cat("\n # of iterations requed =", niter, "  with rmse =", rmse, "\n")
  cat("\n SQUAREM =", SQUAREM, "\n")
  cat(" nSQUARE =", nSQUAREM, ",  minalpha =", minalpha
      , ",  maxalpha =", maxalpha, "\n")
  cat("\nStress 1 = ", stress1, ",   Stress 2 (minimized) = ", stress2, "\n")
  if( print >= 3 ){
   Print(stress1_s, stress2_s)
  }
  Print("\nFinal Config\n", X, W, fmt="8.3 8.3")
  Print(meanX,stdX, corrX, fmt="8.3")
 }


 return( named_list(rmse, stress1, stress2, stress1_s, stress2_s
                    , X, W, vO, vDhat, vD, iter_hist, mlevel, nknots, ndim ) )


} # end of wmdsGN











# generate data
seed=395
seed=56542122
set.seed(seed)

n=20
nG=30

n=10
nG=3

ndim=2
stderr=.2

npair=n*(n-1)/2

X=matrix(rnorm(n*ndim), n, ndim)
X=standard( X )
vX=matrix(t(X),,1)
rownames(X)=paste("s",1:n,sep="")
W=matrix(runif(nG*ndim), nG, ndim)
W=W%*%Diag(1/sqrt(colSums(W^2)))

Print(X,W)
D=distancew( X, W )
vD=vech(D,nodiag=1)


comment('
temp=sqrt(diag(t(vD)%*%vD)/npair)
vD=vD%*%Diag(1/temp)
D=vecsqinv(vD,diag=0, array=1)
W=Diag(1/(temp^2))%*%W
vW=matrix(t(W),,1)
Print(X,W)
D1=distancew( X, W )
Print(D1[,,1],D[,,1])
Print(D1[,,1]/D[,,1])
')

#vD[,2]=(vD[,2])^2
#vD[,3]=vD[,2]^4

temp=sqrt( diag(cov(vD)*(npair-1)/npair) )
vO=vD + matrix(rnorm(npair*nG),npair,nG )%*%Diag(stderr*temp)
vO[vO<0]=0

#temp=Diag(1/sqrt(diag(cov(vO)*(npair-1)/npair)))
#vO=vO%*%temp
O=vechinv(vO, nodiag=1, array=1) # this is n x n x nG

# O=O[,,1]

etime(0)
res1=wmdsGN( O,  ndim=2, mlevel=5, maxiter=400, eps=1e-6, epsd=1e-5, init=1
           , SQUAREM=3, minalpha=-9999, maxalpha=1, estX=1, estW=0
           , print=2 , nSQUAREM=1, nknots=10)
etime(1)


# , W=W, X=X

# Print(X, res1$X, "\n", W,res1$W)
#Print(res1$vDhat, res1$vD, fmt="6.2")

# plot( vO, res1$vDhat )
# plot( vD, res1$vDhat )

comments('
hist1=res1$iter_hist
plot(hist1,type="b")
plot(hist1[3:20,],type="b")
plot(hist1[2:nrow(hist1),],type="b")
')

comments('
etime(0)
res0=wmdsGN( O,  ndim=3, mlevel=4, maxiter=150, eps=1e-9, epsd=1e-9, init=1
           , SQUAREM=0, minalpha=-1.8, maxalpha=-1, estX=1, estW=1
           , print=3 )
etime(1)
')



